@@ -63,7 +63,6 @@ use Net::IP;
 use HTML::Mason;
 use Error qw(:try);
 use Perl6::Junction qw(any);
-use File::Path;
 
 # Module local conf stuff
 use constant DHCPCONFFILE => "/etc/dhcp3/dhcpd.conf";
@@ -89,7 +88,7 @@ sub _create
 {
 	my $class = shift;
 	my $self  = $class->SUPER::_create(name => 'dhcp', 
-									   printableName => 'dhcp server',
+                                           printableName => 'dhcp server',
                                            domain => 'ebox-dhcp',
                                            @_);
 	bless ($self, $class);
@@ -102,23 +101,6 @@ sub domain
 	return 'ebox-dhcp';
 }
 
-# Method: actions
-#
-# 	Override EBox::ServiceModule::ServiceInterface::actions
-#
-sub actions
-{
-	return [ 
-	{
-		'action' => __('Remove dhcpd3 init script links'),
-		'reason' => __('eBox will take care of starting and stopping ' .
-						'the services.'),
-		'module' => 'squid'
-	}
-    ];
-}
-
-
 # Method: usedFiles
 #
 #	Override EBox::ServiceModule::ServiceInterface::usedFiles
@@ -162,6 +144,15 @@ sub serviceModuleName
 	return 'dhcp';
 }
 
+# Method: isRunning
+#
+#   Override EBox::ServiceModule::ServiceInterface::isRunning
+#
+sub isRunning
+{
+    return EBox::Service::running(DHCP_SERVICE);
+}
+
 # Method: _stopService
 #
 #        Stop the dhcp service
@@ -208,57 +199,6 @@ sub statusSummary
 		EBox::Service::running(DHCP_SERVICE), $self->service);
 }
 
-# Method: onInstall
-#
-# 	Static method to execute the first time the module is
-# 	installed.
-#
-sub onInstall
-{
-    EBox::init();
-
-    # Create user conf dir
-    mkdir (CONF_DIR, 0755);
-
-    _addDHCPService();
-    _addTFTPService();
-
-    my $fw = EBox::Global->modInstance('firewall');
-    $fw->setInternalService('dhcp', 'accept');
-    $fw->setInternalService('tftp', 'accept');
-    $fw->saveConfigRecursive();
-}
-
-# Method: onRemove
-#
-# 	Static method to execute before the module is uninstalled
-#
-sub onRemove
-{
-    EBox::init();
-
-    my $serviceMod = EBox::Global->modInstance('services');
-    my $fwMod = EBox::Global->modInstance('firewall');
-
-    if ($serviceMod->serviceExists('name' => 'dhcp')) {
-        $serviceMod->removeService('name' => 'dhcp');
-    } else {
-        EBox::info("Not removing dhcp service as it already removed");
-    }
-
-    if ($serviceMod->serviceExists('name' => 'tftp')) {
-        $serviceMod->removeService('name' => 'tftp');
-    } else {
-        EBox::info("Not removing tftp service as it already removed");
-    }
-
-    $fwMod->saveConfigRecursive();
-
-    # Remove user defined conf dir
-    File::Path::rmtree( CONF_DIR );
-
-}
-
 # Method: menu
 #
 # Overrides:
@@ -374,7 +314,11 @@ sub _exposedMethods
         'removeFixedAddress' => { action   => 'del',
                                   path     => [ 'FixedAddressTable' ],
                                   indexes  => [ 'name' ],
-                                }
+                                },
+        'setLeases' => { action  => 'set',
+                         path    => [ 'LeaseTimes' ],
+                         indexes => [ 'id' ],
+                       },
         );
     return \%methods;
 
@@ -792,8 +736,17 @@ sub ranges # (iface)
 						'value' => $iface);
 	}
 
-        return $self->_getModel('rangeModel', $iface)->printableValueRows();
-
+    my $model = $self->_getModel('rangeModel', $iface);
+    my @ranges;
+    for my $row (@{$model->rows()}) {
+        push (@ranges, { 
+                name => $row->valueByName('name'),
+                from => $row->valueByName('from'),
+                to => $row->valueByName('to')
+                });
+    }
+    
+    return \@ranges;
 }
 
 # Method: fixedAddressAction
@@ -805,10 +758,12 @@ sub ranges # (iface)
 #	iface - String interface name
 #       action - String the action to be performed (add/set/del)
 #
-#       indexValue - String the index element value, it may be the name,
-#       the ip or the mac since they all are unique
+#       indexValue - String the index element value, it may be the
+#       name, the ip or the mac since they all are unique (Only
+#       required for set/del actions)
 #
-#       indexField - String the field name to use as index
+#       indexField - String the field name to use as index (Only
+#       required for set/del actions)
 #
 #	mac - String mac address
 #	ip - String IPv4 address
@@ -903,7 +858,17 @@ sub fixedAddresses # (interface)
 			iface => $iface));
 	}
 
-        return $self->_getModel('fixedAddrModel', $iface)->printableValueRows();
+	my $model = $self->_getModel('fixedAddrModel', $iface);
+	my @addrs;
+	for my $row (@{$model->rows()}) {
+		push (@addrs, { 
+				name => $row->valueByName('name'),
+				ip => $row->valueByName('ip'),
+				mac => $row->valueByName('mac')
+				});
+	}
+
+	return \@addrs;
 }
 
 # Group: Static or class methods
@@ -999,20 +964,29 @@ sub PluginConfDir
 #
 sub ifaceMethodChanged # (iface, old_method, new_method)
 {
-	my ($self, $iface, $old_method, $new_method) = @_;
+    my ($self, $iface, $old_method, $new_method) = @_;
 
-        if ($old_method eq 'static'
-            and $new_method ne 'static') {
-            my $rangeModel = $self->_getModel('rangeModel', $iface);
-            if ( defined ( $rangeModel )) {
-                return 1 if ( $rangeModel->size() > 0);
-            }
-            my $fixedAddrModel = $self->_getModel('fixedAddrModel', $iface);
-            if ( defined ( $fixedAddrModel )) {
-                return 1 if ( $fixedAddrModel->size() > 0);
-            }
+    # Mark managers as changed every time we attempt to change the
+    # iface method from/to static
+    if ($old_method eq 'static' or $new_method eq 'static') {
+        my $manager = EBox::Model::ModelManager->instance();
+        $manager->markAsChanged();
+        $manager = EBox::Model::CompositeManager->Instance();
+        $manager->markAsChanged();
+    }
+
+    if ($old_method eq 'static'
+          and $new_method ne 'static') {
+        my $rangeModel = $self->_getModel('rangeModel', $iface);
+        if ( defined ( $rangeModel )) {
+            return 1 if ( $rangeModel->size() > 0);
         }
-        return 0;
+        my $fixedAddrModel = $self->_getModel('fixedAddrModel', $iface);
+        if ( defined ( $fixedAddrModel )) {
+            return 1 if ( $fixedAddrModel->size() > 0);
+        }
+    }
+    return 0;
 }
 
 # Method: vifaceAdded
@@ -1042,8 +1016,8 @@ sub vifaceAdded # (iface, viface, address, netmask)
         # Check that the new IP for the virtual interface isn't in any range
         foreach my $rangeModel (@rangeModels) {
             foreach my $rangeRow (@{$rangeModel->rows()}) {
-                my $from = $rangeRow->{plainValueHash}->{from};
-                my $to   = $rangeRow->{plainValueHash}->{to};
+                my $from = $rangeRow->valueByName('from');
+                my $to   = $rangeRow->valueByName('to');
                 my $range = new Net::IP($from . ' - ' . $to);
                 unless ( $ip->overlaps($range) == $IP_NO_OVERLAP ) {
                     throw EBox::Exceptions::External(
@@ -1052,7 +1026,7 @@ sub vifaceAdded # (iface, viface, address, netmask)
                             . "DHCP range '{range}' in the interface "
                             . "'{iface}'. Please, remove it before trying "
                             . 'to add a virtual interface using it.',
-                            range => $rangeRow->{plainValueHash}->{name},
+                            range => $rangeRow->valueByName('name'),
                             iface => $rangeModel->index()));
                 }
             }
@@ -1062,7 +1036,7 @@ sub vifaceAdded # (iface, viface, address, netmask)
         # Check the new IP for the virtual interface is not a fixed address
         foreach my $fixedAddrModel (@fixedAddrModels) {
             foreach my $mappingRow (@{$fixedAddrModel->rows()}) {
-                my $fixedIP = new Net::IP($mappingRow->{plainValueHash}->{ip});
+                my $fixedIP = new Net::IP($mappingRow->valueByName('ip'));
                 unless ( $ip->overlaps($fixedIP) == $IP_NO_OVERLAP ) {
                     throw EBox::Exceptions::External(
                            __x('The IP address of the virtual interface '
@@ -1144,9 +1118,9 @@ sub staticIfaceAddressChanged # (iface, old_addr, old_mask, new_addr, new_mask)
         my $manager = EBox::Model::ModelManager->instance();
         my $rangeModel = $manager->model("/dhcp/RangeTable/$iface");
         foreach my $rangeRow (@{$rangeModel->rows()}) {
-            my $range = new Net::IP($rangeRow->{plainValueHash}->{from}
+            my $range = new Net::IP($rangeRow->valueByName('from')
                                     . ' - ' .
-                                    $rangeRow->{plainValueHash}->{to});
+                                    $rangeRow->valueByName('to'));
             # Check the range is still in the network
             unless ($range->overlaps($netIP) == $IP_A_IN_B_OVERLAP){
                 return 1;
@@ -1158,7 +1132,7 @@ sub staticIfaceAddressChanged # (iface, old_addr, old_mask, new_addr, new_mask)
         }
         my $fixedAddrModel = $manager->model("/dhcp/FixedAddressTable/$iface");
         foreach my $mappingRow (@{$fixedAddrModel->rows()}) {
-            my $fixedIP = new Net::IP( $mappingRow->{plainValueHash}->{ip} );
+            my $fixedIP = new Net::IP( $mappingRow->valueByName('ip') );
             # Check the fixed address is still in the network
             unless($fixedIP->overlaps($netIP) == $IP_A_IN_B_OVERLAP){
                 return 1;
@@ -1524,66 +1498,8 @@ sub _thinClientOption # (option, iface)
 
 }
 
-sub _addDHCPService
-{
-	my $serviceMod = EBox::Global->modInstance('services');
-
-	if (not $serviceMod->serviceExists('name' => 'dhcp')) {
-		 $serviceMod->addService('name' => 'dhcp',
-                        'description' => __('Dynamic Host Configuration Protocol'),
-			'protocol' => 'udp',
-			'sourcePort' => 'any',
-			'destinationPort' => 67,
-			'internal' => 1,
-			'readOnly' => 1);
-
-	} else {
-		 $serviceMod->setService('name' => 'dhcp',
-                        'description' => __('Dynamic Host Configuration Protocol'),
-			'protocol' => 'udp',
-			'sourcePort' => 'any',
-			'destinationPort' => 67,
-			'internal' => 1,
-			'readOnly' => 1);
-
-		EBox::info("Not adding dhcp services as it already exists");
-	}
-
-    $serviceMod->save();
-
-}
-
-# Add tftp service on ebox-services module
-sub _addTFTPService
-{
-    my $serviceMod = EBox::Global->modInstance('services');
-
-    if (not $serviceMod->serviceExists('name' => 'tftp')) {
-        $serviceMod->addService('name' => 'tftp',
-                                'description' => __('Trivial File Transfer Protocol'),
-                                'protocol' => 'udp',
-                                'sourcePort' => 'any',
-                                'destinationPort' => 69,
-                                'internal' => 1,
-                                'readOnly' => 1);
-
-    } else {
-        $serviceMod->setService('name' => 'tftp',
-                                'description' => __('Trivial File Transfer Protocol'),
-                                'protocol' => 'udp',
-                                'sourcePort' => 'any',
-                                'destinationPort' => 69,
-                                'internal' => 1,
-                                'readOnly' => 1);
-
-        EBox::info("Not adding tftp service as it already exists");
-    }
-
-    $serviceMod->save();
-
-}
-
 # Configure the firewall rules to add
+# XXX maybe this is dead code?
 sub _configureFirewall {
 	my $self = shift;
 	my $fw = EBox::Global->modInstance('firewall');
@@ -1660,4 +1576,14 @@ sub _nStaticIfaces
 }
 
 
+# Method:  userConfDir
+#
+#  Returns:
+#  path to the user configuration dir
+sub userConfDir
+{
+  return CONF_DIR;
+}
+
+
  1;
